<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * Author: Chandravadana Kandregula
 * Code Release Version: 1.0.0
 * Website: http://www.pinpointdesigns.co.uk
 *
 * Class DeliveryMessageRepository
 * @package Pinpoint\DeliveryMessage\Model;
 */

namespace Pinpoint\DeliveryMessage\Model;

use Exception;
use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\DeliveryMessage\Api\Data;
use Pinpoint\DeliveryMessage\Api\Data\DeliveryMessageInterface;
use Pinpoint\DeliveryMessage\Api\Data\DeliveryMessageInterfaceFactory;
use Pinpoint\DeliveryMessage\Api\DeliveryMessageRepositoryInterface;
use Pinpoint\DeliveryMessage\Model\ResourceModel\DeliveryMessage as ResourceDeliveryMessage;
use Pinpoint\DeliveryMessage\Model\ResourceModel\DeliveryMessage\CollectionFactory as DeliveryMessageCollectionFactory;

class DeliveryMessageRepository implements DeliveryMessageRepositoryInterface
{
    /**
     * @var ResourceDeliveryMessage
     */
    protected $resource;

    /**
     * @var DeliveryMessageFactory
     */
    protected $messageFactory;

    /**
     * @var DeliveryMessageCollectionFactory
     */
    protected $messageCollectionFactory;

    /**
     * @var Data\DeliveryMessageSearchResultsInterfaceFactory
     */
    protected $searchResultsFactory;

    /**
     * @var DataObjectHelper
     */
    protected $dataObjectHelper;

    /**
     * @var DataObjectProcessor
     */
    protected $dataObjectProcessor;

    /**
     * @var DeliveryMessageInterfaceFactory
     */
    protected $dataDeliveryMessageFactory;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @param ResourceDeliveryMessage $resource
     * @param DeliveryMessageFactory $messageFactory
     * @param DeliveryMessageInterfaceFactory $dataDeliveryMessageFactory
     * @param DeliveryMessageCollectionFactory $messageCollectionFactory
     * @param Data\DeliveryMessageSearchResultsInterfaceFactory $searchResultsFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param DataObjectProcessor $dataObjectProcessor
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        ResourceDeliveryMessage $resource,
        DeliveryMessageFactory $messageFactory,
        DeliveryMessageInterfaceFactory $dataDeliveryMessageFactory,
        DeliveryMessageCollectionFactory $messageCollectionFactory,
        Data\DeliveryMessageSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager
    ) {
        $this->resource = $resource;
        $this->messageFactory = $messageFactory;
        $this->messageCollectionFactory = $messageCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->dataDeliveryMessageFactory = $dataDeliveryMessageFactory;
        $this->dataObjectProcessor = $dataObjectProcessor;
        $this->storeManager = $storeManager;
    }

    /**
     * Save DeliveryMessage data
     *
     * @param DeliveryMessageInterface $message
     * @return DeliveryMessageInterface
     * @throws CouldNotSaveException
     */
    public function save(DeliveryMessageInterface $message)
    {
        try {
            $this->resource->save($message);
        } catch (Exception $exception) {
            throw new CouldNotSaveException(__($exception->getMessage()));
        }
        return $message;
    }

    /**
     * Load DeliveryMessage data by given DeliveryMessage Identity
     *
     * @param int $messageId
     * @return DeliveryMessage
     * @throws NoSuchEntityException
     */
    public function getById(int $messageId)
    {
        $message = $this->messageFactory->create();
        $this->resource->load($message, $messageId, 'message_id');
        if (!$message->getId()) {
            throw new NoSuchEntityException(__('The message with the "%1" ID doesn\'t exist.', $messageId));
        }
        return $message;
    }

    /**
     * Delete DeliveryMessage
     *
     * @param DeliveryMessageInterface $message
     * @return bool
     * @throws CouldNotDeleteException
     */
    public function delete(DeliveryMessageInterface $message)
    {
        try {
            $this->resource->delete($message);
        } catch (Exception $exception) {
            throw new CouldNotDeleteException(__($exception->getMessage()));
        }
        return true;
    }

    /**
     * Delete DeliveryMessage by given DeliveryMessage Identity
     *
     * @param int $messageId
     * @return bool
     * @throws CouldNotDeleteException
     * @throws NoSuchEntityException
     */
    public function deleteById(int $messageId)
    {
        return $this->delete($this->getById($messageId));
    }
}
